#!/usr/bin/tclsh

############################################################################
# Modified by Andriy Anishkin (anishkin@icqmail.com) UMCP
#color style by Ximing
color change rgb green2 0.411 0.706 0.18
color change rgb blue 0.176 0.216 0.57
color change rgb red 0.92 0.27 0.27
color Name C green2
color Type C white
color Type O red2
color Type N cyan3
color change rgb cyan 0.608 0.834 0.824
color element C cyan
color Structure {Alpha Helix} cyan2
color Structure 3_10_Helix iceblue
color Structure Pi_Helix cyan3
color Structure Extended_Beta mauve
color Structure Bridge_Beta tan
color Structure Turn silver
color Structure Coil white
# turn on lights 0 and 1
light 0 on
light 1 on
light 2 off
light 3 off
display nearclip set 0
display rendermode GLSL

# position the stage and axes
#axes location lowerleft
axes location off
stage location off

# position and turn on menus
menu main on
menu main     move 5   540
menu animate  move 125 30
menu edit     move 125 225
menu tracker  move 125 520
menu display  move 395 30
menu graphics move 704 139
menu color    move 125 225
menu files    move 570 483
menu molecule move 125 525
menu labels   move 661 29
menu render   move 125 525
menu sequence move 629 0

#################
#
# hbonds D and A
# focus on heavy atoms
#################
atomselect macro hbond_donors {(name N OH2 OW) or (resname ARG and name NE NH1 NH2) or (resname ASN and name ND2) or (resname GLN and name NE2) or (resname HIS and name ND1 NE2) or (resname HSD and name ND1) or (resname HSE and name NE2) or (resname HSP and name ND1 NE2) or (resname LYS and name NZ) or (resname SER and name OG) or (resname THR and name OG1) or (resname TRP and name NE1) or (resname TYR and name OH)}

atomselect macro hbond_acceptors {(name O OH2 OW) or (resname ASN and name OD1) or (resname ASP and name OD1 OD2) or (resname GLN and name OE1) or (resname GLU and name OE1 OE2) or (resname HIS and name ND1 NE2) or (resname HSD and name NE2) or (resname HSE and name ND1) or (resname SER and name OG) or (resname THR and name OG1) or (resname TYR and name OH)}

############

display projection orthographic

# for protein ligand interaction
user add key n {
	#Apply preselected graphical representation
    foreach mol [molinfo list] {
	mol delrep 0 $mol
 	#mol representation NewRibbons 1.800000 6.000000 2.600000 0
	#mol color ResType

	#for protein and nucleic
	mol color Structure
	mol representation NewCartoon 0.400000 10.000000 3.00000 0
	mol selection {protein or nucleic}
	mol material EdgyShiny
	mol addrep $mol

        #for ligand and metal
        mol color Element
        mol representation Licorice 0.300000 12.000000 12.000000
        mol selection {(noh or polarh) and not protein and not nucleic and not water and not resname UNK CL NA}
        mol material EdgyShiny
        mol addrep $mol

        #for residues around ligand
        mol color Type
        mol representation Licorice 0.150000 12.000000 12.000000
        mol selection {(noh or polarh) and same residue as within 3.5 of (not protein and not nucleic and not water)}
        mol material EdgyShiny
        mol addrep $mol

        #for water and CL NA UNK
        mol color Name
        mol representation CPK 1.000000 0.300000 12.000000 12.000000
        mol selection {(noh or polarh) and (water or resname UNK CL NA)}
        mol material EdgyShiny
        mol addrep $mol
    }

}

# flip
proc fx { } {
		# Rotate (flip) protein 180 degrees around x axis
		[atomselect top all] move [transaxis x 180]
}
proc fy { } {
		# Rotate (flip) protein 180 degrees around y axis
		[atomselect top all] move [transaxis y 180]
}
proc fz { } {
		# Rotate (flip) protein 180 degrees around z axis
		[atomselect top all] move [transaxis z 180]
}



proc ss {{molid top}} {
	# Based on the sscache script by Andrew Dalke (dalke@ks.uiuc.edu)
	# Cache secondary structure information for a given molecule

	#VMD  --- start of VMD description block
	#Name:
	# SSCache
	#Synopsis:
	# Automatically stores secondary structure information for animations
	#Version:
	# 1.0
	#Uses VMD version:
	# 1.1
	#Ease of use:
	# 2
	#Procedures:
	# <li>start_sscache molid - start caching the given molecule
	# <li>stop_sscache molid - stop caching
	# <li>reset_sscache - reset the cache
	# <li>sscache - internal function used by trace
	#Description:
	# Calculates and stores the secondary structure assignment for
	# each timestep.  This lets you see how the secondary structure
	# changes over a trajectory.
	# <p>
	# It is turned on with the command "start_sscache> followed by the
	# molecule number of the molecule whose secondary structure should be
	# saved (the default is "top", which gets converted to the correct
	# molecule index).  Whenever the frame for that molecule changes, the
	# procedure "sscache" is called.
	# <p>
	#   "sscache" is the heart of the script.  It checks if a secondary
	# structure definition for the given molecule number and frame already
	# exists in the Tcl array sscache_data(molecule,frame).  If so, it uses
	# the data to redefine the "structure" keyword values (but only for
	# the protein residues).  If not, it calls the secondary structure
	# routine to evaluate the secondary structure based on the new
	# coordinates.  The results are saved in the sscache_data array.
	# <p>
	# Once the secondary structure values are saved, the molecule can be
	# animated rather quickly and the updates can be controlled by the
	# animate form.
	# <p>
	#  To turn off the trace, use the command "stop_sscache", which
	# also takes the molecule number.  There must be one "stop_sscache"
	# for each "start_sscache".  The command "clear_sscache" resets
	# the saved secondary structure data for all the molecules and all the
	# frames.
	#Files:
	# <a href="sscache.vmd">sscache.vmd</a>
	#See also:
	# the VMD user's guide
	#Author:
	# Andrew Dalke &lt;dalke@ks.uiuc.edu&gt;
	#Url:
	# http://www.ks.uiuc.edu/Research/vmd/script_library/sscache/
	#\VMD  --- end of block


	# start the cache for a given molecule
	proc start_sscache {{molid top}} {
	    global sscache_data
	    if {! [string compare $molid top]} {
	set molid [molinfo top]
	    }
	    global vmd_frame
	    # set a trace to detect when an animation frame changes
	    trace variable vmd_frame($molid) w sscache
	    return
	}

	# remove the trace (need one stop for every start)
	proc stop_sscache {{molid top}} {
	    if {! [string compare $molid top]} {
	set molid [molinfo top]
	    }
	    global vmd_frame
	    trace vdelete vmd_frame($molid) w sscache
	    return
	}


	# reset the whole secondary structure data cache
	proc reset_sscache {} {
	    if [info exists sscache_data] {
	        unset sscache_data
	    }
	    return
	}

	# when the frame changes, trace calls this function
	proc sscache {name index op} {
	    # name == vmd_frame
	    # index == molecule id of the newly changed frame
	    # op == w

	    global sscache_data

	    # get the protein CA atoms
	    set sel [atomselect $index "protein name CA"]

	    ## get the new frame number
	    # Tcl doesn't yet have it, but VMD does ...
	    set frame [molinfo $index get frame]

	    # see if the ss data exists in the cache
	    if [info exists sscache_data($index,$frame)] {
		$sel set structure $sscache_data($index,$frame)
		return
	    }

	    # doesn't exist, so (re)calculate it
	    vmd_calculate_structure $index
	    # save the data for next time
	    set sscache_data($index,$frame) [$sel get structure]

	    return
	}
	start_sscache $molid
}

proc lbl {selectionText {labelInfo {resname resid}} {labelPrefix " "} {color red} {size 1}} {;	# Labels each atom in the 'selectionText' with information 'labelInfo', with arbitrary prefix 'labelPrefix', color 'color' and font size 'size' (default 1)
	if {![llength $selectionText]} {set selectionText "name CA"}
	if {![llength $labelInfo]} {set labelInfo {resname resid}}
	if {![llength $labelPrefix]} {set labelPrefix " "}
	if {![llength $color]} {set color red}
	if {![llength $size]} {set size 1}
    set sel [atomselect top $selectionText]
    draw color $color
    foreach coord [$sel get {x y z}] labelContent [$sel get $labelInfo] {
	    # and draw the text
	    draw text $coord "$labelPrefix$labelContent" size $size
    }
}

####################boundbox###############################
proc boundbox {sel} {
      # get the min and max values for each of the directions
      # (I'm not sure if this is the best way ... )
      # set sel [atomselect top all]

      set coords [lsort -real [$sel get x]]
      set minx [lindex $coords 0]
      set maxx [lindex [lsort -real -decreasing $coords] 0]

      set coords [lsort -real [$sel get y]]
      set miny [lindex $coords 0]
      set maxy [lindex [lsort -real -decreasing $coords] 0]

      set coords [lsort -real [$sel get z]]
      set minz [lindex $coords 0]
      set maxz [lindex [lsort -real -decreasing $coords] 0]

      # and draw the lines
      draw materials off
      # line in x
      draw color red
      draw line "$minx $maxy $minz" "$maxx $maxy $minz"
      draw line "$minx $miny $maxz" "$maxx $miny $maxz"
      draw line "$maxx $maxy $maxz" "$minx $maxy $maxz"
      draw line "$minx $miny $minz" "$maxx $miny $minz"
      # line in y
      draw color green
      draw line "$maxx $miny $minz" "$maxx $maxy $minz"
      draw line "$minx $miny $maxz" "$minx $maxy $maxz"
      draw line "$maxx $maxy $maxz" "$maxx $miny $maxz"
      draw line "$minx $miny $minz" "$minx $maxy $minz"
      # line in z
      draw color blue
      draw line "$maxx $miny $minz" "$maxx $miny $maxz"
      draw line "$minx $maxy $minz" "$minx $maxy $maxz"
      draw line "$maxx $maxy $maxz" "$maxx $maxy $minz"
      draw line "$minx $miny $minz" "$minx $miny $maxz"

      #draw color yellow
      #draw line "$minx $miny $minz" "$maxx $maxy $maxz"
      puts "the center is: [expr ($minx + $maxx)/2] [expr ($miny + $maxy)/2] [expr ($minz + $maxz)/2]"
      puts "center of selection:"
      measure center $sel
	  puts "minix and maxx: $minx $maxx"
	  puts "miny and maxy: $miny $maxy"
	  puts "minz and maxz: $minz $maxz"

}

##########################
#                        #
#  for NAMD PME box      #
##########################
#pbcboxsize
proc pbcboxsize {} {
        # Periodic Boundary Conditions
        set sel [atomselect top all]
        set cen [measure center $sel]
        set mm [measure minmax $sel]
        set vmin [lindex $mm 0]
        set vmax [lindex $mm 1]
        set vdif [vecsub $vmax $vmin]
        set xx [lindex $vdif 0]
        set yy [lindex $vdif 1]
        set zz [lindex $vdif 2]
        set cenx [lindex $cen 0]
        set ceny [lindex $cen 1]
        set cenz [lindex $cen 2]
        set padding 0
        # PME (for full-system periodic electrostatics)
        package require pmepot
        set gridsizex [::PMEPot::good_fft_dim $xx]
        set gridsizey [::PMEPot::good_fft_dim $yy]
        set gridsizez [::PMEPot::good_fft_dim $zz]

        puts "cellBasisVector1    [expr $xx+$padding]    0.   0."
        puts "cellBasisVector2     0.   [expr $yy+$padding]   0."
        puts "cellBasisVector3     0.    0   [expr $zz+$padding]"
        puts "cellOrigin          $cenx $ceny $cenz"
        puts "wrapAll             on"
        puts "PME                 yes"
        puts "PMEGridSizeX        $gridsizex"
        puts "PMEGridSizeY        $gridsizey"
        puts "PMEGridSizeZ        $gridsizez"
}


#####################
# draw_bond function#
#####################

#Usage:
# draw_bond -mol1 0 -index1 1 -mol2 1 -index2 1 -h_nbars 25 -h_space 1 -h_arrow 0 -h_radius 0.1 -color yellow -mat Opaque -h_type cylinder -h_resol 25
proc draw_bond_help { } {
	puts "----------draw bond function help information-------------
	#Usage example:
	#draw_bond -mol1 0 -index1 1 -mol2 1 -index2 1 -h_nbars 25 -h_space 1 -h_arrow 0 -h_radius 0.1 -color yellow -mat Opaque -h_type cylinder -h_resol 25
	#--->options:
	# -mol1 0; the first molecule id, the default value is top
	# -index1 200; the first atom index number in mol1
	# -mol2 0; the the molecular id of the second atom
	# -index2; the second atom index number in mol2
	# -h_nbars 25; number of bars
	# -h_space 1; space between bars
	# -h_arrow 0; draw arrows on the two sides of the bond or not
	# -h_radius 0.1; the bond radius
	# -color yellow; the bond color
	# -mat Opaque; the material render method
	# -h_type cylinder; supporting sphere, cylinder, pymol, cone, line
	# -h_reol 25; bond resolution"
}

proc draw_bond {args} {
	#the original function is: draw_bond {res1 res2 {color yellow}}
	#set default values
	#set mol_graphics_id 0

	set mol_id_1 top
	set mol_id_2 top

	set hbonds_nbars 25
	set hbonds_ratio 1
	set hbonds_arrow 0
	set hbonds_radius 0.1


	set color yellow
	set material Opaque
	set h_resol 25

	set h_type cylinder

	#Parse options---------------------------
	# draw_bond -mol1 0 -index1 1 -mol2 1 -index2 1 -h_nbars 25 -h_space 1 -h_arrow 0 -h_radius 0.1 -color yellow -h_type cylinder
	for { set argnum 0 } { $argnum < [llength $args] } { incr argnum } {
		set arg [ lindex $args $argnum ]
		set val [ lindex $args [expr $argnum + 1]]

		switch -- $arg {
	    	"-mol1"      {set mol_id_1 $val; incr argnum}
	    	"-index1"	 {set index1 $val; incr argnum}
	    	"-mol2"	     {set mol_id_2 $val; incr argnum}
	    	"-index2"	 {set index2 $val; incr argnum}
	    	"-h_nbars"   {set hbonds_nbars $val; incr argnum}
	    	"-h_space"   {set hbonds_ratio $val; incr argnum}
	    	"-h_radius"  {set hbonds_radius $val; incr argnum}
	    	"-h_arrow"   {set hbonds_arrow $val; incr argnum}
	    	"-color"     {set color $val; incr argnum}
	    	"-mat"       {set material $val; incr argnum}
	    	"-h_resol"   {set h_resol $val; incr argnum}
	    	"-h_type"    {set h_type $val; incr argnum}

	    	default { puts "unknown option: $arg and you must set the -index1 and index2 values "; return }
		}
    }
	#-----------stop parse-------------------
	lassign [[atomselect $mol_id_1 "index $index1"] get {x y z}] START
	lassign [[atomselect $mol_id_2 "index $index2"] get {x y z}] END
	set nbars $hbonds_nbars
	set X1 [lindex $START 0] ; set X2 [lindex $END 0]
	set Y1 [lindex $START 1] ; set Y2 [lindex $END 1]
	set Z1 [lindex $START 2] ; set Z2 [lindex $END 2]
	if { $X1 != "" && $X2 != "" && $Y1 != "" && $Y2 != "" && $Z1 != "" && $Z2 != "" } {
		set drawarrows $hbonds_arrow
		set LX [expr ( $X2 - $X1 )]
		set LY [expr ( $Y2 - $Y1 )]
		set LZ [expr ( $Z2 - $Z1 )]

		if { $drawarrows != 0 } {
			incr nbars 2
			set imin 1
		} else {
			set imin 0
		}

		#graphics $mol_graphics_id color $color
		draw color $color
		draw material $material
		set line_radius [expr ($hbonds_radius * 20)]

		set barx [expr ($hbonds_ratio * $LX) / ($nbars * ($hbonds_ratio + 1) - 1)]
		set bary [expr ($hbonds_ratio * $LY) / ($nbars * ($hbonds_ratio + 1) - 1)]
		set barz [expr ($hbonds_ratio * $LZ) / ($nbars * ($hbonds_ratio + 1) - 1)]

		set gapx [expr ($barx / $hbonds_ratio)]
		set gapy [expr ($bary / $hbonds_ratio)]
		set gapz [expr ($barz / $hbonds_ratio)]

		for {set i $imin} { $i < [expr $nbars - $imin]} { incr i 1} {
			set DEBX [expr ($X1 + ($i * ($barx + $gapx)))]
			set ENDX [expr ($DEBX + $barx)]
			set DEBY [expr ($Y1 + ($i * ($bary + $gapy)))]
			set ENDY [expr ($DEBY + $bary)]
			set DEBZ [expr ($Z1 + ($i * ($barz + $gapz)))]
			set ENDZ [expr ($DEBZ + $barz)]

			if { $h_type == "sphere" } {
				draw sphere "$DEBX $DEBY $DEBZ" resolution $h_resol radius $hbonds_radius
			} elseif {$h_type == "cone"} {
				draw cone "$DEBX $DEBY $DEBZ" "$ENDX $ENDY $ENDZ" resolution $h_resol radius $hbonds_radius
			} elseif {$h_type == "line"} {
				draw line $START $END style dashed width $line_radius
			} elseif {$h_type == "pymol"} {
				draw cylinder "$DEBX $DEBY $DEBZ" "$ENDX $ENDY $ENDZ" resolution $h_resol radius $hbonds_radius filled yes
				draw sphere "$DEBX $DEBY $DEBZ" resolution $h_resol radius $hbonds_radius
				draw sphere "$ENDX $ENDY $ENDZ" resolution $h_resol radius $hbonds_radius
			} else {
				draw cylinder "$DEBX $DEBY $DEBZ" "$ENDX $ENDY $ENDZ" resolution $h_resol radius $hbonds_radius filled yes
			}

			if { $i eq 1 && $drawarrows } {
				lassign "$DEBX $DEBY $DEBZ" ARROWHEADX ARROWHEADY ARROWHEADZ
			}
			if { $i eq [expr $nbars - 2] && $drawarrows } {
				lassign "$ENDX $ENDY $ENDZ" ARROWTAILX ARROWTAILY ARROWTAILZ
			}
			unset DEBX DEBY DEBZ ENDX ENDY ENDZ
		}

		if { $drawarrows == 2 } {

			draw cone "$ARROWHEADX $ARROWHEADY $ARROWHEADZ" "$X1 $Y1 $Z1" radius $hbonds_radius resolution $h_resol
			draw cone "$ARROWTAILX $ARROWTAILY $ARROWTAILZ" "$X2 $Y2 $Z2" radius $hbonds_radius resolution $h_resol

			unset ARROWHEADX ARROWHEADY ARROWHEADZ ARROWTAILX ARROWTAILY ARROWTAILZ
		} elseif {$drawarrows == 1} {
		    draw cone "$ARROWHEADX $ARROWHEADY $ARROWHEADZ" "$X1 $Y1 $Z1" radius $hbonds_radius resolution $h_resol
		    unset ARROWHEADX ARROWHEADY ARROWHEADZ ARROWTAILX ARROWTAILY ARROWTAILZ

		} elseif {$drawarrows == -1} {
		    draw cone "$ARROWTAILX $ARROWTAILY $ARROWTAILZ" "$X2 $Y2 $Z2" radius $hbonds_radius resolution $h_resol
		    unset ARROWHEADX ARROWHEADY ARROWHEADZ ARROWTAILX ARROWTAILY ARROWTAILZ
		}
		unset X1 X2 Y1 Y2 Z1 Z2
		unset LX LY LZ barx bary barz gapx gapy gapz line_radius
	}
}

########################
# Pickbond
# Author: Justin Gullingsrud
# Date: 11 January, 2001
# Compatible VMD versions: Post-1.6; i.e, anything built after the above
#    date ;-)  You can rewrite the delbond and addbond procs below to reflect
#    the current state of VMD's bond changing API; it's undergone a bit of flux.

# This is a script to allow users to add or delete bonds by picking atoms
# with the mouse.  Type "pickbond add" at the text console, and now picking
# pairs of atoms will result a new bond being created between them.
# "pickbond del" will cause bonds between picked pairs of atoms to be
# deleted.  "pickbond stop" ceases all pickbond behavior.


proc delbond { molid atom1 atom2 } {
  if { $atom1 > $atom2 } {
    set tmp $atom1
    set atom1 $atom2
    set atom2 $tmp
  }
  set sel [atomselect $molid "index $atom1 $atom2"]
  lassign [$sel getbonds] bond1 bond2

  set id [lsearch -exact $bond1 $atom2]
  if { $id != -1 } {
    set bond1 [lreplace $bond1 $id $id ]
  }
  set id [lsearch -exact $bond2 $atom1]
  if { $id != -1 } {
    set bond2 [lreplace $bond2 $id $id ]
  }
  $sel setbonds [list $bond1 $bond2]
  $sel delete
}

proc addbond { molid atom1 atom2 } {
  if {$atom1 == $atom2 } {
    #error "cannot bond an atom to itself"
    return
  }
  if { $atom1 > $atom2 } {
    set tmp $atom1
    set atom1 $atom2
    set atom2 $tmp
  }
  set sel [atomselect $molid "index $atom1 $atom2"]
  lassign [$sel getbonds] bond1 bond2

  set id [lsearch -exact $bond1 $atom2]
  if { $id == -1 } {
    lappend bond1 $atom2
  }
  set id [lsearch -exact $bond2 $atom1]
  if { $id == -1 } {
    lappend bond2 $atom1
  }
  $sel setbonds [list $bond1 $bond2]
  $sel delete
}

# set this proc to trace on vmd_pick_atom

set pickbond_molid -1
set pickbond_atom1 -1
set pickbond_proc addbond

proc pickbond_trace { name1 name2 op } {
  global pickbond_proc pickbond_molid pickbond_atom1 vmd_pick_mol vmd_pick_atom

  if { $pickbond_molid == -1 } {
    # this is the first atom
    set pickbond_molid $vmd_pick_mol
    set pickbond_atom1 $vmd_pick_atom
  } else {
    # we have two atoms: call the proc
    catch {$pickbond_proc $pickbond_molid $pickbond_atom1 $vmd_pick_atom}
    set pickbond_molid -1
    set pickbond_atom1 -1
  }
}


#
# pickbond [add | del | stop ]
#
proc pickbond { op } {
  global pickbond_molid pickbond_atom1 pickbond_proc

  set pickbond_molid -1
  set pickbond_atom1 -1

  uplevel {trace vdelete vmd_pick_atom w pickbond_trace}

  if { $op == "add" } {
    set pickbond_proc addbond
    uplevel {trace variable vmd_pick_atom w pickbond_trace}
  } elseif { $op == "del" } {
    set pickbond_proc delbond
    uplevel {trace variable vmd_pick_atom w pickbond_trace}
  } elseif { $op == "stop" } {
    puts "No more pickbond"
  } else {
    error "Unknown option: choose one of add, del, or stop"
  }
}

##############
#Ledock boxer
##############
proc boxer {args} {
  set nargs [llength $args]
  puts $nargs
  if { $nargs == 1 } {
        puts "Generate a box from selection"
        set sel [lindex $args 0]
        puts "your selection is $sel"

        # get the min and max values for each of the directions
        set coords [lsort -real [$sel get x]]
        set minx [lindex $coords 0]
        set maxx [lindex [lsort -real -decreasing $coords] 0]

        set coords [lsort -real [$sel get y]]
        set miny [lindex $coords 0]
        set maxy [lindex [lsort -real -decreasing $coords] 0]

        set coords [lsort -real [$sel get z]]
        set minz [lindex $coords 0]
        set maxz [lindex [lsort -real -decreasing $coords] 0]

    } elseif  {$nargs == 6} {
        set minx [lindex $args 0]
        set maxx [lindex $args 1]
        set miny [lindex $args 2]
        set maxy [lindex $args 3]
        set minz [lindex $args 4]
        set maxz [lindex $args 5]

    } else {
        puts "I'm sure your input has some problem, and you will have some errors like:"
    }

   # and draw the lines
   draw materials off
   # line in x direction
   draw color red
   draw line "$minx $maxy $minz" "$maxx $maxy $minz"
   draw line "$minx $miny $maxz" "$maxx $miny $maxz"
   draw line "$maxx $maxy $maxz" "$minx $maxy $maxz"
   draw line "$minx $miny $minz" "$maxx $miny $minz"
   # line in y direction
   draw color green
   draw line "$maxx $miny $minz" "$maxx $maxy $minz"
   draw line "$minx $miny $maxz" "$minx $maxy $maxz"
   draw line "$maxx $maxy $maxz" "$maxx $miny $maxz"
   draw line "$minx $miny $minz" "$minx $maxy $minz"
   # line in z direction
   draw color blue
   draw line "$maxx $miny $minz" "$maxx $miny $maxz"
   draw line "$minx $maxy $minz" "$minx $maxy $maxz"
   draw line "$maxx $maxy $maxz" "$maxx $maxy $minz"
   draw line "$minx $miny $minz" "$minx $miny $maxz"
   # print result
	 puts  "------FOR LEDOCK------------------------------"
   puts  "xmin and xmax: [format %7.2f%7.2f $minx $maxx]"
   puts  "ymin and ymax: [format %7.2f%7.2f $miny $maxy]"
   puts  "zmin and zmax: [format %7.2f%7.2f $minz $maxz]"

   puts  "[format %-7.2f%-7.2f $minx $maxx]"
   puts  "[format %-7.2f%-7.2f $miny $maxy]"
   puts  "[format %-7.2f%-7.2f $minz $maxz]"
   #print center and size
   set centx [expr {($maxx + $minx)/2}]
   set centy [expr {($maxy + $miny)/2}]
   set centz [expr {($maxz + $minz)/2}]
   set sizex [expr {$maxx - $minx}]
   set sizey [expr {$maxy - $miny}]
   set sizez [expr {$maxz - $minz}]
	 puts "---------FOR VINA------------------------------"
   puts "center_x = [format %5.2f $centx]"
   puts "center_y = [format %5.2f $centy]"
   puts "center_z = [format %5.2f $centz]"
   puts "size_x   = [format %5.2f $sizex]"
   puts "size_y   = [format %5.2f $sizey]"
   puts "size_z   = [format %5.2f $sizez]"

}

#######
#Geom_center
########################
proc geom_center {selection} {
        # set the geometrical center to 0
        set gc [veczero]
        # [$selection get {x y z}] returns a list of {x y z}
        #    values (one per atoms) so get each term one by one
        foreach coord [$selection get {x y z}] {
           # sum up the coordinates
           set gc [vecadd $gc $coord]
        }
        # and scale by the inverse of the number of atoms
        return [vecscale [expr 1.0 /[$selection num]] $gc]
}






#################
#
#draw hbonds between two selections
# focus on heavy atoms
#################
# draw_hbonds: draw cylinders between hydrogen and acceptor atoms in hydrogen bonds
# formed between two selections. Cutoff and angle are also specified.
proc draw_hbonds {molid cutoff angle sel1text sel2text} {
  set a [atomselect $molid $sel1text]
  set b [atomselect $molid $sel2text]
  #set coords [[atomselect $molid all] get {x y z}]

  # get hbonds with 1 as donor and 2 as acceptor
  foreach { d1 a1 h1 } [measure hbonds $cutoff $angle $a $b] break
  # get hbonds wiith 2 as donor and 1 as acceptor
  foreach { d2 a2 h2 } [measure hbonds $cutoff $angle $b $a] break
  # combine the lists
  set dhh [concat $d1 $d2]
  set hyd [concat $h1 $h2]
  set acc [concat $a1 $a2]
  # draw them!
  draw delete all
  #foreach hi $hyd ai $acc {
  #  set hpos [lindex $coords $hi]
  #  set apos [lindex $coords $ai]
  #  draw color yellow
  #  draw cylinder $hpos $apos radius 0.2
  #}
  foreach di $dhh ai $acc {
    #set dpos [lindex $coords $di]
    #set apos [lindex $coords $ai]
    #draw color yellow
    #draw cylinder $dpos $apos radius 0.2
    draw_bond -mol1 $molid -index1 $di -mol2 0 -index2 $ai -h_nbars 7 -h_space 1 -h_arrow 0 -h_radius 0.15 -color yellow -h_type cylinder

  }
}

#  draw_hbonds top 3.0 45 {chain A} {chain B}


###################
# rotating a bond #
###################
#rotating bond
#mvdih {selected atoms} index1 index2 degree
#index1 is in the selected atoms, and index2 is the atom linked to index1
#example: mvdih {resid 34 and index 404 to 409} 404 402 50
proc mvdih {moveselection a1 a2 rotatedeg} {
  set mvselect [atomselect top $moveselection]
  set anchor1 [atomselect top "index $a1"]
  set bonded2 [atomselect top "index $a2"]
  $mvselect move [trans bond [lindex [$anchor1 get {x y z}] 0] [lindex [$bonded2 get {x y z}] 0] $rotatedeg deg]
}

##################
#measure distance#
##################
# measure_distance index1 index2
proc measure_distance {xxx yyy} {
  veclength [vecsub [lindex [[atomselect top "index $xxx"] get {x y z}] 0] [lindex [[atomselect top "index $yyy"] get {x y z}] 0]]
 }


##################
#measure angle#
##################

# compute angle between three arbitrary points in space
# (initially has been written for hydrogen bond angles,
# notation has been kept: D-H...A)
# usage: angle {x1 y1 z1} {x2 y2 z2} {x3 y3 z3}
proc angle {D H A} {
  # cosine of the angle between three points
  # cos = ( v1 * v2 ) / |v1| * |v2|, where v1 and v2 are vectors
  # get Pi 3.14159265358979323846
  global M_PI
  # initialize arrays
  set d() 0; set h() 0; set a() 0; set hd() 0; set ha() 0;
  # split coordinates
  set d(x) [lindex $D 0]; set d(y) [lindex $D 1]; set d(z) [lindex $D 2];
  set h(x) [lindex $H 0]; set h(y) [lindex $H 1]; set h(z) [lindex $H 2];
  set a(x) [lindex $A 0]; set a(y) [lindex $A 1]; set a(z) [lindex $A 2];
  # setup vectors hd and ha
  set hd(x) [expr $d(x) - $h(x)];
  set hd(y) [expr $d(y) - $h(y)];
  set hd(z) [expr $d(z) - $h(z)];
  set ha(x) [expr $a(x) - $h(x)];
  set ha(y) [expr $a(y) - $h(y)];
  set ha(z) [expr $a(z) - $h(z)];
  # compute cosine
  set cosine [expr \
    ($hd(x)*$ha(x) + $hd(y)*$ha(y) + $hd(z)*$ha(z)) / \
    (sqrt(pow($hd(x),2) + pow($hd(y),2) + pow($hd(z),2)) * \
     sqrt(pow($ha(x),2) + pow($ha(y),2) + pow($ha(z),2)))];
  # convert cosine to angle in degrees
  set angle [expr acos($cosine)*(180.0/$M_PI)]
  return $angle;
}


################
#cat dcd
#################
proc catpdb {outfile args} {

    if { [catch {open ${outfile}.pdb w} out] } {
	puts stderr "Could not open ${outfile}.pdb for writing"
    }

    if {[llength $args] == 0} {
	error "Missing list of pdb files to merge.
usage:
catpdb <outfile> <pdblist>"
    }


    set firstfile true

    foreach pdbfile $args {

	if { [catch {open $pdbfile r} inp] } {
	    puts stderr "Could not open ${pdbfile} for reading"
	}

	puts "opened $pdbfile"

	set pdb [split [read $inp] "\n"]

	foreach line $pdb {
	    # do some line processing here

	    if {$firstfile} {
		puts $out $line
		set firstfile false
	    }

	    if {[regexp "^ATOM.*" $line]} {
		puts $out $line
	    } else {
		continue
	    }

	}
	vmdcon -info "catpdb: pdbfile done"
	close $inp

    }
    puts $out "END"
    close $out
}

####################
# RMSD TRAJ        #
####################
# Prints the RMSD of the protein atoms between each timestep
# and the first timestep for the given molecule id (default: top)
proc rmsd_traj {{mol top}} {
        # use frame 0 for the reference
        set reference [atomselect $mol "backbone" frame 0]
        # the frame being compared
        set compare [atomselect $mol "backbone"]
        set all [atomselect $mol all]

        set num_steps [molinfo $mol get numframes]
        for {set frame 0} {$frame < $num_steps} {incr frame} {
                # get the correct frame
                $compare frame $frame
                $all frame $frame

                # compute the transformation
                set trans_mat [measure fit $compare $reference]
                # do the alignment
                $all move $trans_mat
                # compute the RMSD
                set rmsd [measure rmsd $compare $reference]
                # print the RMSD
                puts "RMSD: $frame  $rmsd"
        }
}
